package explorviz.live_trace_processing.filter.reduction.evaluation.metrics_based;

import java.util.ArrayList;
import java.util.List;

import explorviz.live_trace_processing.filter.IPipeReceiver;
import explorviz.live_trace_processing.filter.reduction.AbstractReductionFilter;
import explorviz.live_trace_processing.record.event.AbstractAfterEventRecord;
import explorviz.live_trace_processing.record.event.AbstractAfterFailedEventRecord;
import explorviz.live_trace_processing.record.event.AbstractBeforeEventRecord;
import explorviz.live_trace_processing.record.event.AbstractEventRecord;
import explorviz.live_trace_processing.record.event.AbstractRemoteEventRecord;
import explorviz.live_trace_processing.record.trace.Trace;

public class StackDepthFilter extends AbstractReductionFilter {

	private final int maxStackDepth;

	public StackDepthFilter(final IPipeReceiver receiver, final int maxStackDepth) {
		super(receiver, "StackDepthLimitation");
		this.maxStackDepth = maxStackDepth;
	}

	@Override
	protected Trace reduceTrace(final Trace trace) {
		final List<AbstractEventRecord> newTraceEvents = new ArrayList<AbstractEventRecord>();
		int count = 0;
		for (final AbstractEventRecord record : trace.getTraceEvents()) {
			if (record instanceof AbstractBeforeEventRecord) {
				if (count < maxStackDepth) {
					newTraceEvents.add(record);
				}
				count++;
			} else if ((record instanceof AbstractAfterEventRecord)
					|| (record instanceof AbstractAfterFailedEventRecord)) {
				count--;
				if (count < maxStackDepth) {
					newTraceEvents.add(record);
				}
			} else if (record instanceof AbstractRemoteEventRecord) {
				newTraceEvents.add(record);
			} else {
				newTraceEvents.add(record);
			}
		}
		return new Trace(newTraceEvents, true);
	}

}
